import { DependencyContainer } from "tsyringe";

import { IPostDBLoadMod } from "@spt-aki/models/external/IPostDBLoadMod";
import { DatabaseServer } from "@spt-aki/servers/DatabaseServer";
import { ILogger } from "@spt-aki/models/spt/utils/ILogger";
import { LogTextColor } from "@spt-aki/models/spt/logging/LogTextColor";

import Config from "../config/config.json";

class Mod implements IPostDBLoadMod
{
    public logger: ILogger;

    public postDBLoad(container: DependencyContainer): void 
    {
        // If the mod is not enabled end it
        if (!Config.ISENABLED) return;

        //#region variables and some initialization
        this.logger = container.resolve("WinstonLogger");
        const armbandColors: Record<string,string> = {
            "blue":"5b3f3af486f774679e752c1f",
            "green":"5b3f3b0186f774021a2afef7",
            "red":"5b3f3ade86f7746b6b790d8e",
            "white":"5b3f16c486f7747c327f55f7",
            "yellow":"5b3f3b0e86f7746752107cda",
            "purple":"5f9949d869e2777a0e779ba5"
        }
        let scavArmband:string;
        let raiderArmband:string;
        let rogueArmband:string;
        let usecArmband:string;
        let bearArmband:string;
        let bossArmband:string;
        //#endregion

        //#region Checking for config proper config option, killing the mod if it's improper.
        switch (Config.SCAVARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                scavArmband = armbandColors[Config.SCAVARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, SCAVARMBAND must be a valid color.");
        }
        switch (Config.RAIDERARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                raiderArmband = armbandColors[Config.RAIDERARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, RAIDERARMBAND must be a valid color.");
        }
        switch (Config.ROGUEARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                rogueArmband = armbandColors[Config.ROGUEARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, ROGUEARMBAND must be a valid color.");
        }
        switch (Config.USECARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                usecArmband = armbandColors[Config.USECARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, USECRMBAND must be a valid color.");
        }
        switch (Config.BEARARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                bearArmband = armbandColors[Config.BEARARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, BEARARMBAND must be a valid color.");
        }
        switch (Config.BOSSARMBAND.toLowerCase()){
            case "blue":
            case "green":
            case "red":
            case "white":
            case "yellow":
            case "purple":
                bossArmband = armbandColors[Config.BOSSARMBAND];
                break;
            default:
                return this.logger.error("[FactionIdentifierArmbands] Error in config/config.json, BOSSRMBAND must be a valid color.");
        }
        //#endregion VALIDATION    

        // get database from server
        const databaseServer = container.resolve<DatabaseServer>("DatabaseServer");

        // Get all the in-memory json found in /assets/database
        const tables = databaseServer.getTables();

        // Get all the bots and add armbands to them
        /* all the bot types here:
        BOSS: bossBully, bossGluhar, bossKilla, bossKojaniy, bossSanitar, bossTagilla, sectantPriest, bossKnight
        BOSS ESCORT: followerBully, followerKojaniy, followerSanitar, followerGluharSnipe, followerGluharAssault, followerGluharScout, followerGluharSecurity, sectantWarrior, followerBirdEye, followerBigPipe, followertagilla
        SCAV: assault, marksman, cursedAssault
        RAIDER: pmcBot, exUsec
        PMC BOT: assaultGroup, usec, bear
        */
        if (Config.LOGGING.toLowerCase().match("basic") || Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] Adding Armbands to bots...", LogTextColor.BLUE);
        let counter = 0;

        for (const botType in tables.bots.types)
        {
            if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] current botType is: " + botType, LogTextColor.BLUE);
            const bot = tables.bots.types[botType];

            switch (botType){
                case "assault":
                case "marksman":
                case "cursedassault":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {scavArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.SCAVARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;    
                case "pmcbot":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {raiderArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.RAIDERARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;
                case "exusec":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {rogueArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.ROGUEARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;
                case "assaultgroup":
                case "usec":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {usecArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.USECARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;
                case "bear":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {bearArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.BEARARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;
                case "bossbully":
                case "bossgluhar":
                case "bosskilla":
                case "bosskojaniy":
                case "bosssanitar":
                case "bosstagilla":
                case "bossknight":
                case "followerbigpipe":
                case "followerbirdeye":
                case "followerbully":
                case "followergluharassault":
                case "followergluharscout":
                case "followergluharsecurity":
                case "followergluharsnipe":
                case "followerkojaniy":
                case "followersanitar":
                case "followertagilla":
                case "sectantpriest":
                case "sectantwarrior":
                    bot.chances.equipment.ArmBand = 100;
                    bot.inventory.equipment.ArmBand = {bossArmband:1};
                    if (Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] added "+ Config.BOSSARMBAND +" to " + botType, LogTextColor.BLUE);
                    counter++;
                    break;
            }
        }
        if (Config.LOGGING.toLowerCase().match("basic") || Config.LOGGING.toLowerCase().match("verbose")) this.logger.logWithColor("[FactionIdentifierArmbands] Finished adding "+counter+" Armbands to bots.", LogTextColor.BLUE);
    }
}

module.exports = { mod: new Mod() }